@@ -1,26 +1,19 @@
-# -*- coding: utf-8 -*-
 ########################################################################
 #
-#       License: MIT
 #       Created: September 22, 2010
-#       Author:  Francesc Alted - faltet@gmail.com
+#       Author:  The Blosc development team - blosc@blosc.org
 #
 ########################################################################
 
 import os
+import pickle
+import subprocess
 import sys
-from distutils.version import LooseVersion
-try:
-    import cPickle as pickle
-except ImportError:
-    import pickle
+from ._version import LooseVersion
 
 from blosc import blosc_extension as _ext
 import blosc
 
-# version number hack
-vi = sys.version_info
-
 
 def detect_number_of_cores():
     """
@@ -42,9 +35,14 @@ def detect_number_of_cores():
             if isinstance(ncpus, int) and ncpus > 0:
                 return ncpus
         else:  # OSX:
-            return int(os.popen2("sysctl -n hw.ncpu")[1].read())
+            completed = subprocess.run(("sysctl", "-n", "hw.ncpu"),
+                                       stdout=subprocess.PIPE)
+            if completed.returncode == 0:
+                ncpus = int(completed.stdout)
+                if ncpus > 0:
+                    return ncpus
     # Windows:
-    if "NUMBER_OF_PROCESSORS" in os.environ:
+    elif "NUMBER_OF_PROCESSORS" in os.environ:
         ncpus = int(os.environ["NUMBER_OF_PROCESSORS"])
         if ncpus > 0:
             return ncpus
@@ -92,6 +90,7 @@ def set_nthreads(nthreads):
         raise ValueError("the number of threads cannot be larger than %d" %
                          blosc.MAX_THREADS)
 
+    blosc.nthreads = nthreads
     return _ext.set_nthreads(nthreads)
 
 
@@ -104,7 +103,7 @@ def set_blocksize(blocksize):
     Notes
     -----
 
-    This is a low-level function and is recommened for expert users only.
+    This is a low-level function and is recommended for expert users only.
 
     Examples
     --------
@@ -340,7 +339,7 @@ def free_resources():
 
 
 def _check_shuffle(shuffle):
-    if shuffle not in [blosc.NOSHUFFLE, blosc.SHUFFLE, blosc.BITSHUFFLE]:
+    if shuffle not in (blosc.NOSHUFFLE, blosc.SHUFFLE, blosc.BITSHUFFLE):
         raise ValueError("shuffle can only be one of NOSHUFFLE, SHUFFLE"
                          " and BITSHUFFLE.")
     if (shuffle == blosc.BITSHUFFLE and
@@ -373,6 +372,13 @@ def _check_bytesobj(bytesobj):
                         "supported as input")
 
 
+def _check_byteslike(bytes_like):
+    try:
+        memoryview(bytes_like)
+    except Exception:
+        raise TypeError("Input type %s must be a bytes-like object that supports Python Buffer Protocol" % type(bytes_like))
+
+
 def _check_input_length(input_name, input_len):
     if input_len > blosc.MAX_BUFFERSIZE:
         raise ValueError("%s cannot be larger than %d bytes" %
@@ -427,9 +433,9 @@ def compress(bytesobj, typesize=8, clevel=9, shuffle=blosc.SHUFFLE,
     Examples
     --------
 
-    >>> import array
+    >>> import array, sys
     >>> a = array.array('i', range(1000*1000))
-    >>> a_bytesobj = a.tostring()
+    >>> a_bytesobj = a.tobytes()
     >>> c_bytesobj = blosc.compress(a_bytesobj, typesize=4)
     >>> len(c_bytesobj) < len(a_bytesobj)
     True
@@ -538,15 +544,17 @@ def compress_ptr(address, items, typesize=8, clevel=9, shuffle=blosc.SHUFFLE,
     return _ext.compress_ptr(address, length, typesize, clevel, shuffle, cname)
 
 
-def decompress(bytesobj, as_bytearray=False):
-    """decompress(bytesobj)
+def decompress(bytes_like, as_bytearray=False):
+    """decompress(bytes_like)
 
-    Decompresses a bytesobj compressed object.
+    Decompresses a bytes-like compressed object.
 
     Parameters
     ----------
-    bytesobj : str / bytes
-        The data to be decompressed.
+    bytes_like : bytes-like object
+        The data to be decompressed.  Must be a bytes-like object
+        that supports the Python Buffer Protocol, like bytes, bytearray,
+        memoryview, or numpy.ndarray.
     as_bytearray : bool, optional
         If this flag is True then the return type will be a bytearray object
         instead of a bytesobject.
@@ -561,14 +569,14 @@ def decompress(bytesobj, as_bytearray=False):
     Raises
     ------
     TypeError
-        If bytesobj is not of type bytes or string.
+        If bytes_like does not support Buffer Protocol
 
     Examples
     --------
 
-    >>> import array
+    >>> import array, sys
     >>> a = array.array('i', range(1000*1000))
-    >>> a_bytesobj = a.tostring()
+    >>> a_bytesobj = a.tobytes()
     >>> c_bytesobj = blosc.compress(a_bytesobj, typesize=4)
     >>> a_bytesobj2 = blosc.decompress(c_bytesobj)
     >>> a_bytesobj == a_bytesobj2
@@ -583,20 +591,22 @@ def decompress(bytesobj, as_bytearray=False):
 
     """
 
-    return _ext.decompress(bytesobj, as_bytearray)
+    return _ext.decompress(bytes_like, as_bytearray)
 
 
-def decompress_ptr(bytesobj, address):
-    """decompress_ptr(bytesobj, address)
+def decompress_ptr(bytes_like, address):
+    """decompress_ptr(bytes_like, address)
 
-    Decompresses a bytesobj compressed object into the memory at address.
+    Decompresses a bytes_like compressed object into the memory at address.
 
     Parameters
     ----------
-    bytesobj : str / bytes
-        The data to be decompressed.
+    bytes_like : bytes-like object
+        The data to be decompressed. Must be a bytes-like object
+        that supports the Python Buffer Protocol, like bytes, bytearray,
+        memoryview, or numpy.ndarray.
     address : int or long
-        the pointer to the data to be compressed
+        The address at which to write the decompressed data
 
     Returns
     -------
@@ -653,10 +663,10 @@ def decompress_ptr(bytesobj, address):
 
     """
 
-    _check_bytesobj(bytesobj)
+    _check_byteslike(bytes_like)
     _check_address(address)
 
-    return _ext.decompress_ptr(bytesobj, address)
+    return _ext.decompress_ptr(bytes_like, address)
 
 
 def pack_array(array, clevel=9, shuffle=blosc.SHUFFLE, cname='blosclz'):
@@ -795,17 +805,16 @@ def load_tests(loader, tests, pattern):
 def os_release_pretty_name():
     for p in ('/etc/os-release', '/usr/lib/os-release'):
         try:
-            f = open(p, 'rt')
+            f = open(p)
             for line in f:
                 name, _, value = line.rstrip().partition('=')
                 if name == 'PRETTY_NAME':
                     if len(value) >= 2 and value[0] in '"\'' and value[0] == value[-1]:
                         value = value[1:-1]
                     return value
-        except IOError:
+        except OSError:
             pass
-    else:
-        return None
+    return None
 
 def print_versions():
     """Print all the versions of software that python-blosc relies on."""
